@@ -16,17 +16,18 @@
 #define CEPH_BUFFER_H
 
 #if defined(__linux__)
+#include <stdlib.h>
 #include <linux/types.h>
 #elif defined(__FreeBSD__)
 #include <sys/types.h>
 #include "include/inttypes.h"
+#include <stdlib.h>
 #endif
 
 #ifndef _XOPEN_SOURCE
 # define _XOPEN_SOURCE 600
 #endif
 
-#include <stdlib.h>
 #include <stdio.h>
 
 #ifdef DARWIN
@@ -422,7 +423,7 @@ public:
     __u32 crc32c(__u32 crc) {
       for (std::list<ptr>::const_iterator it = _buffers.begin(); 
 	   it != _buffers.end(); 
-	   it++)
+	   ++it)
 	if (it->length())
 	  crc = ceph_crc32c_le(crc, (unsigned char*)it->c_str(), it->length());
       return crc;
