@@ -115,7 +115,7 @@ namespace librbd {
 
   int RBD::create(IoCtx& io_ctx, const char *name, uint64_t size, int *order)
   {
-    return librbd::create(io_ctx, name, size, true, 0, order, 0, 0);
+    return librbd::create(io_ctx, name, size, order);
   }
 
   int RBD::create2(IoCtx& io_ctx, const char *name, uint64_t size,
@@ -128,7 +128,8 @@ namespace librbd {
 		   uint64_t features, int *order, uint64_t stripe_unit,
 		   uint64_t stripe_count)
   {
-    return librbd::create(io_ctx, name, size, false, features, order, stripe_unit, stripe_count);
+    return librbd::create(io_ctx, name, size, false, features, order,
+			  stripe_unit, stripe_count);
   }
 
   int RBD::clone(IoCtx& p_ioctx, const char *p_name, const char *p_snap_name,
@@ -181,6 +182,12 @@ namespace librbd {
     c->rbd_comp = this;
   }
 
+  bool RBD::AioCompletion::is_complete()
+  {
+    librbd::AioCompletion *c = (librbd::AioCompletion *)pc;
+    return c->is_complete();
+  }
+
   int RBD::AioCompletion::wait_for_complete()
   {
     librbd::AioCompletion *c = (librbd::AioCompletion *)pc;
@@ -408,6 +415,12 @@ namespace librbd {
     return librbd::snap_list(ictx, snaps);
   }
 
+  bool Image::snap_exists(const char *snap_name)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    return librbd::snap_exists(ictx, snap_name);
+  }
+
   int Image::snap_set(const char *snap_name)
   {
     ImageCtx *ictx = (ImageCtx *)ctx;
@@ -430,6 +443,26 @@ namespace librbd {
     return librbd::read_iterate(ictx, ofs, len, cb, arg);
   }
 
+  int Image::read_iterate2(uint64_t ofs, uint64_t len,
+			      int (*cb)(uint64_t, size_t, const char *, void *),
+			      void *arg)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    int64_t r = librbd::read_iterate(ictx, ofs, len, cb, arg);
+    if (r > 0)
+      r = 0;
+    return (int)r;
+  }
+
+  int Image::diff_iterate(const char *fromsnapname,
+			  uint64_t ofs, uint64_t len,
+			  int (*cb)(uint64_t, size_t, int, void *),
+			  void *arg)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    return librbd::diff_iterate(ictx, fromsnapname, ofs, len, cb, arg);
+  }
+
   ssize_t Image::write(uint64_t ofs, size_t len, bufferlist& bl)
   {
     ImageCtx *ictx = (ImageCtx *)ctx;
@@ -475,6 +508,18 @@ namespace librbd {
     return librbd::flush(ictx);
   }
 
+  int Image::aio_flush(RBD::AioCompletion *c)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    return librbd::aio_flush(ictx, (librbd::AioCompletion *)c->pc);
+  }
+
+  int Image::invalidate_cache()
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    return librbd::invalidate_cache(ictx);
+  }
+
 } // namespace librbd
 
 extern "C" void rbd_version(int *major, int *minor, int *extra)
@@ -521,7 +566,7 @@ extern "C" int rbd_create(rados_ioctx_t p, const char *name, uint64_t size, int
 {
   librados::IoCtx io_ctx;
   librados::IoCtx::from_rados_ioctx_t(p, io_ctx);
-  return librbd::create(io_ctx, name, size, true, 0, order, 0, 0);
+  return librbd::create(io_ctx, name, size, order);
 }
 
 extern "C" int rbd_create2(rados_ioctx_t p, const char *name,
@@ -540,7 +585,8 @@ extern "C" int rbd_create3(rados_ioctx_t p, const char *name,
 {
   librados::IoCtx io_ctx;
   librados::IoCtx::from_rados_ioctx_t(p, io_ctx);
-  return librbd::create(io_ctx, name, size, false, features, order, stripe_unit, stripe_count);
+  return librbd::create(io_ctx, name, size, false, features, order,
+			stripe_unit, stripe_count);
 }
 
 extern "C" int rbd_clone(rados_ioctx_t p_ioctx, const char *p_name,
@@ -654,7 +700,8 @@ extern "C" int rbd_open(rados_ioctx_t p, const char *name, rbd_image_t *image,
   librbd::ImageCtx *ictx = new librbd::ImageCtx(name, "", snap_name, io_ctx,
 						false);
   int r = librbd::open_image(ictx);
-  *image = (rbd_image_t)ictx;
+  if (r >= 0)
+    *image = (rbd_image_t)ictx;
   return r;
 }
 
@@ -666,7 +713,8 @@ extern "C" int rbd_open_read_only(rados_ioctx_t p, const char *name,
   librbd::ImageCtx *ictx = new librbd::ImageCtx(name, "", snap_name, io_ctx,
 						true);
   int r = librbd::open_image(ictx);
-  *image = (rbd_image_t)ictx;
+  if (r >= 0)
+    *image = (rbd_image_t)ictx;
   return r;
 }
 
@@ -1005,6 +1053,27 @@ extern "C" int64_t rbd_read_iterate(rbd_image_t image, uint64_t ofs, size_t len,
   return librbd::read_iterate(ictx, ofs, len, cb, arg);
 }
 
+extern "C" int rbd_read_iterate2(rbd_image_t image, uint64_t ofs, uint64_t len,
+				 int (*cb)(uint64_t, size_t, const char *, void *),
+				 void *arg)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  int64_t r = librbd::read_iterate(ictx, ofs, len, cb, arg);
+  if (r > 0)
+    r = 0;
+  return (int)r;
+}
+
+extern "C" int rbd_diff_iterate(rbd_image_t image,
+				const char *fromsnapname,
+				uint64_t ofs, uint64_t len,
+				int (*cb)(uint64_t, size_t, int, void *),
+				void *arg)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  return librbd::diff_iterate(ictx, fromsnapname, ofs, len, cb, arg);
+}
+
 extern "C" ssize_t rbd_write(rbd_image_t image, uint64_t ofs, size_t len,
 			     const char *buf)
 {
@@ -1060,6 +1129,25 @@ extern "C" int rbd_flush(rbd_image_t image)
   return librbd::flush(ictx);
 }
 
+extern "C" int rbd_aio_flush(rbd_image_t image, rbd_completion_t c)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
+  return librbd::aio_flush(ictx, (librbd::AioCompletion *)comp->pc);
+}
+
+extern "C" int rbd_invalidate_cache(rbd_image_t image)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  return librbd::invalidate_cache(ictx);
+}
+
+extern "C" int rbd_aio_is_complete(rbd_completion_t c)
+{
+  librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
+  return comp->is_complete();
+}
+
 extern "C" int rbd_aio_wait_for_complete(rbd_completion_t c)
 {
   librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
