@@ -74,8 +74,8 @@ int FileJournal::_open(bool forwrite, bool create)
   fd = TEMP_FAILURE_RETRY(::open(fn.c_str(), flags, 0644));
   if (fd < 0) {
     int err = errno;
-    dout(2) << "FileJournal::_open: unable to open journal: open() failed: "
-	    << cpp_strerror(err) << dendl;
+    dout(2) << "FileJournal::_open unable to open journal "
+	    << fn << ": " << cpp_strerror(err) << dendl;
     return -err;
   }
 
@@ -87,10 +87,16 @@ int FileJournal::_open(bool forwrite, bool create)
     goto out_fd;
   }
 
-  if (S_ISBLK(st.st_mode))
+  if (S_ISBLK(st.st_mode)) {
     ret = _open_block_device();
-  else
+  } else {
+    if (aio && !force_aio) {
+      derr << "FileJournal::_open: disabling aio for non-block journal.  Use "
+	   << "journal_force_aio to force use of aio anyway" << dendl;
+      aio = false;
+    }
     ret = _open_file(st.st_size, st.st_blksize, create);
+  }
 
   if (ret)
     goto out_fd;
@@ -293,11 +299,10 @@ int FileJournal::_open_file(int64_t oldsize, blksize_t blksize,
       return -err;
     }
     ret = ::posix_fallocate(fd, 0, newsize);
-    if (ret < 0) {
-      int err = errno;
+    if (ret) {
       derr << "FileJournal::_open_file : unable to preallocation journal to "
-	   << newsize << " bytes: " << cpp_strerror(err) << dendl;
-      return -err;
+	   << newsize << " bytes: " << cpp_strerror(ret) << dendl;
+      return -ret;
     }
     max_size = newsize;
   }
@@ -387,7 +392,10 @@ int FileJournal::create()
     header.alignment = block_size;
   else
     header.alignment = 16;  // at least stay word aligned on 64bit machines...
+
   header.start = get_top();
+  header.start_seq = 0;
+
   print_header();
 
   // static zeroed buffer for alignment padding
@@ -519,7 +527,7 @@ int FileJournal::open(uint64_t fs_op_seq)
 
   // find next entry
   read_pos = header.start;
-  uint64_t seq = 0;
+  uint64_t seq = header.start_seq;
   while (1) {
     bufferlist bl;
     off64_t old_pos = read_pos;
@@ -940,7 +948,6 @@ void FileJournal::align_bl(off64_t pos, bufferlist& bl)
 
 int FileJournal::write_bl(off64_t& pos, bufferlist& bl)
 {
-  align_bl(pos, bl);
   int ret;
 
   off64_t spos = ::lseek64(fd, pos, SEEK_SET);
@@ -955,6 +962,8 @@ int FileJournal::write_bl(off64_t& pos, bufferlist& bl)
     return ret;
   }
   pos += bl.length();
+  if (pos == header.max_size)
+    pos = get_top();
   return 0;
 }
 
@@ -976,8 +985,6 @@ void FileJournal::do_write(bufferlist& bl)
     hbp = prepare_header();
   }
 
-  write_lock.Unlock();
-
   dout(15) << "do_write writing " << write_pos << "~" << bl.length() 
 	   << (hbp.length() ? " + header":"")
 	   << dendl;
@@ -987,6 +994,14 @@ void FileJournal::do_write(bufferlist& bl)
   // entry
   off64_t pos = write_pos;
 
+  // Adjust write_pos
+  align_bl(pos, bl);
+  write_pos += bl.length();
+  if (write_pos >= header.max_size)
+    write_pos = write_pos - header.max_size + get_top();
+
+  write_lock.Unlock();
+
   // split?
   off64_t split = 0;
   if (pos + bl.length() > header.max_size) {
@@ -1003,13 +1018,12 @@ void FileJournal::do_write(bufferlist& bl)
 	   << ") failed" << dendl;
       ceph_abort();
     }
-    assert(pos == header.max_size);
+    assert(pos == get_top());
     if (hbp.length()) {
       // be sneaky: include the header in the second fragment
       second.push_front(hbp);
       pos = 0;          // we included the header
-    } else
-      pos = get_top();  // no header, start after that
+    }
     if (write_bl(pos, second)) {
       derr << "FileJournal::do_write: write_bl(pos=" << pos
 	   << ") failed" << dendl;
@@ -1064,10 +1078,7 @@ void FileJournal::do_write(bufferlist& bl)
 
   write_lock.Lock();    
 
-  // wrap if we hit the end of the journal
-  if (pos == header.max_size)
-    pos = get_top();
-  write_pos = pos;
+  assert(write_pos == pos);
   assert(write_pos % header.alignment == 0);
 
   {
@@ -1477,7 +1488,7 @@ void FileJournal::pop_write()
   writeq.pop_front();
 }
 
-void FileJournal::commit_start()
+void FileJournal::commit_start(uint64_t seq)
 {
   dout(10) << "commit_start" << dendl;
 
@@ -1487,8 +1498,18 @@ void FileJournal::commit_start()
     break; // all good
 
   case FULL_FULL:
-    dout(1) << " FULL_FULL -> FULL_WAIT.  last commit epoch committed, waiting for a new one to start." << dendl;
-    full_state = FULL_WAIT;
+    if (seq >= journaled_seq) {
+      dout(1) << " FULL_FULL -> FULL_WAIT.  commit_start on seq "
+	      << seq << " > journaled_seq " << journaled_seq
+	      << ", moving to FULL_WAIT."
+	      << dendl;
+      full_state = FULL_WAIT;
+    } else {
+      dout(1) << "FULL_FULL commit_start on seq "
+	      << seq << " < journaled_seq " << journaled_seq
+	      << ", remaining in FULL_FULL"
+	      << dendl;
+    }
     break;
 
   case FULL_WAIT:
@@ -1516,29 +1537,31 @@ void FileJournal::committed_thru(uint64_t seq)
   dout(5) << "committed_thru " << seq << " (last_committed_seq " << last_committed_seq << ")" << dendl;
   last_committed_seq = seq;
 
+  // completions!
+  {
+    Mutex::Locker locker(finisher_lock);
+    queue_completions_thru(seq);
+    if (plug_journal_completions && seq >= header.start_seq) {
+      dout(10) << " removing completion plug, queuing completions thru journaled_seq " << journaled_seq << dendl;
+      plug_journal_completions = false;
+      queue_completions_thru(journaled_seq);
+    }
+  }
+
   // adjust start pointer
   while (!journalq.empty() && journalq.front().first <= seq) {
     journalq.pop_front();
   }
   if (!journalq.empty()) {
     header.start = journalq.front().second;
+    header.start_seq = journalq.front().first;
   } else {
     header.start = write_pos;
+    header.start_seq = seq + 1;
   }
   must_write_header = true;
   print_header();
 
-  {
-    Mutex::Locker locker(finisher_lock);
-    // completions!
-    queue_completions_thru(seq);
-    if (plug_journal_completions) {
-      dout(10) << " removing completion plug, queuing completions thru journaled_seq " << journaled_seq << dendl;
-      plug_journal_completions = false;
-      queue_completions_thru(journaled_seq);
-    }
-  }
-
   // committed but unjournaled items
   while (!writeq_empty() && peek_write().seq <= seq) {
     dout(15) << " dropping committed but unwritten seq " << peek_write().seq 
@@ -1573,9 +1596,12 @@ void FileJournal::put_throttle(uint64_t ops, uint64_t bytes)
   }
 }
 
-void FileJournal::make_writeable()
+int FileJournal::make_writeable()
 {
-  _open(true);
+  dout(10) << __func__ << dendl;
+  int r = _open(true);
+  if (r < 0)
+    return r;
 
   if (read_pos > 0)
     write_pos = read_pos;
@@ -1585,6 +1611,7 @@ void FileJournal::make_writeable()
 
   must_write_header = true;
   start_writer();
+  return 0;
 }
 
 void FileJournal::wrap_read_bl(
@@ -1662,39 +1689,6 @@ bool FileJournal::read_entry(
   }
 
   stringstream errss;
-  while (result == MAYBE_CORRUPT &&
-	 (static_cast<uint64_t>(pos - read_pos) <
-	  g_conf->journal_max_corrupt_search)) {
-    errss << "Entry at pos " << pos << " possibly corrupt due to: ("
-	  << ss.str() << ")" << std::endl;
-    ss.str(string());
-    ss.clear();
-    pos = next_pos;
-    result = do_read_entry(
-      pos,
-      &next_pos,
-      &bl,
-      &seq,
-      &ss);
-  }
-
-  if (result == SUCCESS) {
-    if (seq >= next_seq) {
-      derr << errss.str() << dendl;
-      derr << "Entry at pos " << pos << " valid, there are missing sequence "
-	   << "numbers prior to seq " << seq << dendl;
-      if (g_conf->journal_ignore_corruption) {
-	if (corrupt)
-	  *corrupt = true;
-	return false;
-      } else {
-	assert(0);
-      }
-    } else { // We read a valid, but old entry, no problem
-      return false;
-    }
-  }
-
   if (seq < header.committed_up_to) {
     derr << "Unable to read past sequence " << seq
 	 << " but header indicates the journal has committed up through "
