@@ -82,6 +82,10 @@ TEST_F(PGLogTest, rewind_divergent_log) {
     hobject_t divergent_object;
     eversion_t divergent_version;
     eversion_t newhead;
+
+    hobject_t divergent;
+    divergent.hash = 0x9;
+
     {
       pg_log_entry_t e;
 
@@ -90,16 +94,16 @@ TEST_F(PGLogTest, rewind_divergent_log) {
       log.tail = e.version;
       log.log.push_back(e);
       e.version = newhead = eversion_t(1, 4);
-      e.soid.hash = 0x9;
+      e.soid = divergent;
       e.op = pg_log_entry_t::MODIFY;
       log.log.push_back(e);
-      log.index();
       e.version = divergent_version = eversion_t(1, 5);
-      e.soid.hash = 0x9;
+      e.soid = divergent;
       divergent_object = e.soid;
       e.op = pg_log_entry_t::DELETE;
       log.log.push_back(e);
       log.head = e.version;
+      log.index();
 
       info.last_update = log.head;
       info.last_complete = log.head;
@@ -118,6 +122,7 @@ TEST_F(PGLogTest, rewind_divergent_log) {
     rewind_divergent_log(t, newhead, info, remove_snap,
 			 dirty_info, dirty_big_info);
 
+    EXPECT_TRUE(log.objects.count(divergent));
     EXPECT_TRUE(missing.is_missing(divergent_object));
     EXPECT_EQ(1U, log.objects.count(divergent_object));
     EXPECT_EQ(2U, log.log.size());
@@ -668,7 +673,7 @@ TEST_F(PGLogTest, merge_log) {
     bool dirty_info = false;
     bool dirty_big_info = false;
 
-    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1);
+    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1, "");
     info.last_backfill = last_backfill;
     eversion_t stat_version(10, 1);
     info.stats.version = stat_version;
@@ -701,7 +706,7 @@ TEST_F(PGLogTest, merge_log) {
   }
 
   // head and tail match, last_backfill is not set: info.stats is
-  // copied from oinfo.stats but info.stats.reported is guaranteed to
+  // copied from oinfo.stats but info.stats.reported_* is guaranteed to
   // never be replaced by a lower version
   {
     clear();
@@ -717,16 +722,18 @@ TEST_F(PGLogTest, merge_log) {
 
     eversion_t stat_version(10, 1);
     oinfo.stats.version = stat_version;
-    eversion_t stat_reported(10, 1);
-    info.stats.reported = stat_reported;
-    oinfo.stats.reported = eversion_t(1, 1);
+    info.stats.reported_seq = 1;
+    info.stats.reported_epoch = 10;
+    oinfo.stats.reported_seq = 1;
+    oinfo.stats.reported_epoch = 1;
     log.tail = olog.tail = eversion_t(1, 1);
     log.head = olog.head = eversion_t(2, 1);
 
     EXPECT_FALSE(missing.have_missing());
     EXPECT_EQ(0U, log.log.size());
     EXPECT_EQ(eversion_t(), info.stats.version);
-    EXPECT_EQ(stat_reported, info.stats.reported);
+    EXPECT_EQ(1ull, info.stats.reported_seq);
+    EXPECT_EQ(10u, info.stats.reported_epoch);
     EXPECT_TRUE(remove_snap.empty());
     EXPECT_TRUE(t.empty());
     EXPECT_TRUE(info.last_backfill.is_max());
@@ -741,7 +748,8 @@ TEST_F(PGLogTest, merge_log) {
     EXPECT_FALSE(missing.have_missing());
     EXPECT_EQ(0U, log.log.size());
     EXPECT_EQ(stat_version, info.stats.version);
-    EXPECT_EQ(stat_reported, info.stats.reported);
+    EXPECT_EQ(1ull, info.stats.reported_seq);
+    EXPECT_EQ(10u, info.stats.reported_epoch);
     EXPECT_TRUE(remove_snap.empty());
     EXPECT_TRUE(t.empty());
     EXPECT_TRUE(info.purged_snaps.empty());
@@ -823,7 +831,7 @@ TEST_F(PGLogTest, merge_log) {
       olog.head = e.version;
     }
 
-    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1);
+    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1, "");
     info.last_backfill = last_backfill;
     eversion_t stat_version(10, 1);
     info.stats.version = stat_version;
@@ -1032,7 +1040,7 @@ TEST_F(PGLogTest, merge_log) {
       olog.head = e.version;
     }
 
-    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1);
+    hobject_t last_backfill(object_t("oname"), string("key"), 1, 234, 1, "");
     info.last_backfill = last_backfill;
     eversion_t stat_version(10, 1);
     info.stats.version = stat_version;
